@@ -19,6 +19,7 @@ import tempfile
 import textwrap
 import time
 
+from . import configfile
 from . import dput
 from .helper import dputhelper
 
@@ -419,8 +420,16 @@ def upload_stolen_from_dput_main(
 def dcut():
     options, arguments = getoptions()
     if options['debug']:
-        sys.stdout.write('D: calling dput.read_configs\n')
-    config = dput.read_configs(options['config'], options['debug'])
+        sys.stdout.write('D: reading program configuration\n')
+    try:
+        config_files = list(configfile.active_config_files(
+                options['config'], options['debug']))
+        config = dput.read_configs(config_files, options['debug'])
+    except configfile.ConfigurationError as exc:
+        sys.stderr.write("Error: {0}\n".format(str(exc)))
+        sys.stderr.write("Failed to read program configuration; aborting.\n")
+        sys.exit(1)
+
     if (
             not options['host']
             and config.has_option('DEFAULT', 'default_host_main')):
@@ -527,7 +536,7 @@ def create_commands(options, config, parse_changes):
     return removecommands
 
 
-# Copyright © 2015–2021 Ben Finney <bignose@debian.org>
+# Copyright © 2015–2022 Ben Finney <bignose@debian.org>
 # Copyright © 2008–2013 Y Giridhar Appaji Nag <appaji@debian.org>
 # Copyright © 2004–2009 Thomas Viehmann <tv@beamnet.de>
 # Copyright © 2000–2004 Christian Kurz <shorty@debian.org>
