@@ -22,6 +22,7 @@ import pkg_resources
 import testscenarios
 import testtools
 
+import dput.configfile
 import dput.dcut
 from dput.helper import dputhelper
 
@@ -57,6 +58,7 @@ from .test_changesfile import (
         set_changes_file_scenario,
         )
 from .test_configfile import (
+        patch_active_config_files,
         set_config,
         )
 from .test_dputhelper import (
@@ -1324,7 +1326,14 @@ class upload_SimulateTestCase(
                     'simulate': True,
                     },
                 }),
-            ('simulate three-files', {
+            ('simulate files-one', {
+                'config_default_login': "login-from-config-default",
+                'test_args_extra': {
+                    'simulate': True,
+                    },
+                'files_to_upload': [tempfile.mktemp()],
+                }),
+            ('simulate files-three', {
                 'config_default_login': "login-from-config-default",
                 'test_args_extra': {
                     'simulate': True,
@@ -1511,12 +1520,14 @@ class upload_UploadMethodTestCase(
             ]
 
     files_scenarios = [
-            ('no-files', {
+            ('files-none', {
                 'files_to_remove': [],
                 }),
-            ('three-files', {
-                'files_to_remove': [
-                        tempfile.mktemp() for __ in range(3)],
+            ('files-one', {
+                'files_to_remove': [tempfile.mktemp()],
+                }),
+            ('files-three', {
+                'files_to_remove': [tempfile.mktemp() for __ in range(3)],
                 }),
             ]
 
@@ -1541,6 +1552,8 @@ class upload_UploadMethodTestCase(
 class dcut_TestCase(testtools.TestCase):
     """ Base for test cases for `dput` function. """
 
+    function_to_test = staticmethod(dput.dcut.dcut)
+
     def setUp(self):
         """ Set up test fixtures. """
         super().setUp()
@@ -1559,6 +1572,7 @@ class dcut_TestCase(testtools.TestCase):
         self.set_test_args()
 
         patch_getoptions(self)
+        patch_active_config_files(self)
         test_dput_main.patch_parse_changes(self)
         test_dput_main.patch_read_configs(self)
         test_dput_main.set_upload_methods(self)
@@ -1606,11 +1620,11 @@ class dcut_DebugMessageTestCase(dcut_TestCase):
     """ Test cases for `dcut` debug messages. """
 
     def test_emits_debug_message_for_read_configs(self):
-        """ Should emit debug message for `read_configs` call. """
+        """ Should emit debug message to begin reading configuration. """
         self.getoptions_opts['debug'] = True
-        dput.dcut.dcut(**self.test_args)
+        self.function_to_test(**self.test_args)
         expected_output = textwrap.dedent("""\
-                D: calling dput.read_configs
+                D: reading program configuration
                 """)
         self.assertIn(expected_output, sys.stdout.getvalue())
 
@@ -1618,24 +1632,70 @@ class dcut_DebugMessageTestCase(dcut_TestCase):
 class dcut_ConfigFileTestCase(
         testscenarios.WithScenarios,
         dcut_TestCase):
-    """ Test cases for `main` specification of configuration file. """
+    """ Test cases for `dcut` specification of configuration file. """
 
     scenarios = [
             ('default', {
-                'expected_args': (None, unittest.mock.ANY),
+                'expected_config_path': None,
                 }),
             ('config-from-command-line', {
                 'getoptions_opts': {
                     'config': "lorem.conf",
                     },
-                'expected_args': ("lorem.conf", unittest.mock.ANY),
+                'expected_config_path': "lorem.conf",
                 }),
             ]
 
+    def test_calls_active_config_files_with_expected_args(self):
+        """ Should call `active_config_files` with expected arguments. """
+        expected_args = (self.expected_config_path, unittest.mock.ANY)
+        self.function_to_test(**self.test_args)
+        dput.configfile.active_config_files.assert_called_with(*expected_args)
+
     def test_calls_read_configs_with_expected_args(self):
         """ Should call `read_configs` with expected arguments. """
-        dput.dcut.dcut(**self.test_args)
-        dput.dput.read_configs.assert_called_with(*self.expected_args)
+        expected_config_files = self.fake_config_files
+        expected_args = (expected_config_files, unittest.mock.ANY)
+        self.function_to_test(**self.test_args)
+        dput.dput.read_configs.assert_called_with(*expected_args)
+
+    def test_calls_sys_exit_if_configuration_error_when_opening_files(self):
+        """
+        Should call `sys.exit` if `ConfigurationError` from opening files.
+        """
+        set_config(self, 'all-not-exist')
+        self.set_test_args()
+        fake_error_text = "Could not open any configfile, tried ‘foo’, ‘bar’"
+        fake_error = dput.configfile.ConfigurationError(fake_error_text)
+        dput.configfile.active_config_files.side_effect = fake_error
+        with testtools.ExpectedException(FakeSystemExit):
+            self.function_to_test(**self.test_args)
+        expected_output = textwrap.dedent("""\
+                ...{error_text}...
+                """.format(error_text=fake_error_text))
+        self.assertThat(
+                sys.stderr.getvalue(),
+                testtools.matchers.DocTestMatches(
+                    expected_output, flags=doctest.ELLIPSIS))
+        sys.exit.assert_called_with(EXIT_STATUS_FAILURE)
+
+    def test_calls_sys_exit_if_configuration_error_when_reading_files(self):
+        """
+        Should call `sys.exit` if `ConfigurationError` from reading files.
+        """
+        fake_error_text = "Hidden Illuminati messages in config file"
+        fake_error = dput.configfile.ConfigurationError(fake_error_text)
+        dput.dput.read_configs.side_effect = fake_error
+        with testtools.ExpectedException(FakeSystemExit):
+            self.function_to_test(**self.test_args)
+        expected_output = textwrap.dedent("""\
+                ...{error_text}...
+                """.format(error_text=fake_error_text))
+        self.assertThat(
+                sys.stderr.getvalue(),
+                testtools.matchers.DocTestMatches(
+                    expected_output, flags=doctest.ELLIPSIS))
+        sys.exit.assert_called_with(EXIT_STATUS_FAILURE)
 
 
 class dcut_OptionsErrorTestCase(
@@ -1680,7 +1740,7 @@ class dcut_OptionsErrorTestCase(
     def test_emits_expected_error_message(self):
         """ Should emit expected error message. """
         try:
-            dput.dcut.dcut(**self.test_args)
+            self.function_to_test(**self.test_args)
         except FakeSystemExit:
             pass
         self.assertIn(self.expected_output, sys.stdout.getvalue())
@@ -1688,7 +1748,7 @@ class dcut_OptionsErrorTestCase(
     def test_calls_sys_exit_with_failure_exit_status(self):
         """ Should call `sys.exit` with failure exit status. """
         with testtools.ExpectedException(FakeSystemExit):
-            dput.dcut.dcut(**self.test_args)
+            self.function_to_test(**self.test_args)
         sys.exit.assert_called_with(self.expected_exit_status)
 
 
@@ -1735,12 +1795,12 @@ class dcut_NamedHostTestCase(
     def test_emits_debug_message_for_discovered_host(self):
         """ Should emit debug message for discovered host values. """
         self.getoptions_opts['debug'] = True
-        dput.dcut.dcut(**self.test_args)
+        self.function_to_test(**self.test_args)
         self.assertIn(self.expected_debug_output, sys.stdout.getvalue())
 
     def test_calls_write_commands_with_expected_host_option(self):
         """ Should call `write_commands` with expected `host` option. """
-        dput.dcut.dcut(**self.test_args)
+        self.function_to_test(**self.test_args)
         self.assertEqual(1, len(dput.dcut.write_commands.mock_calls))
         (__, call_args, call_kwargs) = dput.dcut.write_commands.mock_calls[0]
         (__, options, __, __) = call_args
@@ -1772,7 +1832,7 @@ class dcut_FileToUploadNameTestCase(
 
     def test_emits_error_message_for_bad_filename(self):
         """ Should emit error message for bad filename. """
-        dput.dcut.dcut(**self.test_args)
+        self.function_to_test(**self.test_args)
         if getattr(self, 'expected_output', None):
             self.assertIn(self.expected_output, sys.stdout.getvalue())
         else:
@@ -1828,7 +1888,7 @@ class dcut_ParseChangesTestCase(
 
     def test_calls_create_commands_with_expected_args(self):
         """ Should call `create_commands` as expected. """
-        dput.dcut.dcut(**self.test_args)
+        self.function_to_test(**self.test_args)
         (expected_options, __) = dput.dcut.getoptions()
         expected_config = self.runtime_config_parser
         expected_parse_changes = dput.dput.parse_changes
@@ -1841,7 +1901,7 @@ class dcut_ParseChangesTestCase(
     def test_calls_write_commands_with_expected_args(self):
         """ Should call `write_commands` as expected. """
         expected_commands = dput.dcut.create_commands.return_value
-        dput.dcut.dcut(**self.test_args)
+        self.function_to_test(**self.test_args)
         (expected_options, __) = dput.dcut.getoptions()
         expected_config = self.runtime_config_parser
         expected_tempdir = getattr(
@@ -1904,7 +1964,7 @@ class dcut_ParseCommandsFromArgumentsTestCase(
 
     def test_calls_parse_queuecommands_with_expected_args(self):
         """ Should call `parse_queuecommands` with expected args. """
-        dput.dcut.dcut(**self.test_args)
+        self.function_to_test(**self.test_args)
         (expected_options, expected_arguments) = dput.dcut.getoptions()
         expected_config = self.runtime_config_parser
         if self.expect_parse_queuecommands:
@@ -1916,7 +1976,7 @@ class dcut_ParseCommandsFromArgumentsTestCase(
     def test_calls_write_commands_with_expected_args(self):
         """ Should call `write_commands` with expected args. """
         expected_commands = dput.dcut.parse_queuecommands.return_value
-        dput.dcut.dcut(**self.test_args)
+        self.function_to_test(**self.test_args)
         (expected_options, __) = dput.dcut.getoptions()
         expected_config = self.runtime_config_parser
         expected_tempdir = getattr(
@@ -1993,7 +2053,7 @@ class dcut_CleanupTestCase(
 
     def test_removes_temporary_directory(self):
         """ Should remove directory `tempdir` at end. """
-        dput.dcut.dcut(**self.test_args)
+        self.function_to_test(**self.test_args)
         shutil.rmtree.assert_called_with(
                 self.tempfile_mkdtemp_file_double.path)
 
@@ -2003,14 +2063,14 @@ class dcut_CleanupTestCase(
         upload_error = RuntimeError("Bad stuff happened")
         upload_method_func.side_effect = upload_error
         try:
-            dput.dcut.dcut(**self.test_args)
+            self.function_to_test(**self.test_args)
         except self.upload_error.__class__:
             pass
         shutil.rmtree.assert_called_with(
                 self.tempfile_mkdtemp_file_double.path)
 
 
-# Copyright © 2015–2018 Ben Finney <bignose@debian.org>
+# Copyright © 2015–2022 Ben Finney <bignose@debian.org>
 #
 # This is free software: you may copy, modify, and/or distribute this work
 # under the terms of the GNU General Public License as published by the
